# @(#) $Header: report.awk,v 1.11 92/01/27 03:14:12 leres Exp $ (LBL)
#
# syslog.awk - produce raw data from netnews syslog files

BEGIN {
	t = "TOTALS"
	a_min_article = 0	# skip newsgroups with zero articles
	a_name_n = 0
	a_name_max = 100
	j_min_group = 0		# skip junked newsgroups with zero articles
	j_name_n = 0
	j_name_max = 30
	j_min_peer = 0		# skip junked peers with zero articles
	j_pname_n = 0
	j_pname_max = 30
	j_min_bogon = 0		# skip bogons with zero
	j_bname_n = 0
	j_bname_max = 10
}

$1 == "in" {
	servers[$2] = $3
	s_received[$2] = $4	# put it in our news spool
	s_refused[$2] = $5	# already had it
	s_rejected[$2] = $6	# wanted it but it was bogus
	s_elapsed[$2] = $7
	next
}

$1 == "out" {
	peers[$2] = 1
#	p_offer[$2] = $3	# XXX nntplink reports a bogus value for this
	p_accept[$2] = $4	# they took it
	p_reject[$2] = $5	# they already had it
	p_failed[$2] = $6	# they wanted it but something went wrong
	p_connects[$2] = $7
	p_user[$2] = $8
	p_system[$2] = $9
	p_elapsed[$2] = $10
	next
}

$1 == "trans" {
	peers[$2] = 1
	t_ns[$2] = $3
	t_timeout[$2] = $4
	t_refused[$2] = $5
	t_discont[$2] = $6
	t_denied[$2] = $7
	next
}

$1 == "read" {
	readers[$2] = 1
	r_connects[$2] = $3
	r_article[$2] = $4
	r_timeout[$2] = $5
	r_post[$2] = $6
	r_group[$2] = $7
	r_user[$2] = $8
	r_system[$2] = $9
	r_elapsed[$2] = $10
	next
}

$1 == "explore" {
	explorers[$2] = $3
	e_denied[$2] = $4
	next
}

$1 == "bogon" {
	h = $2
	bogon = $3
	if (bogon > j_min_bogon) {
		bogons[h] = bogon
		# Insert in order
		j_bname_n++
		for (i = 1; i <= j_bname_n; i++) {
			if (bogons[j_bname[i]] < bogon) {
				# Insert before this guy
				for (j = j_bname_n; j >= i; j--)
					j_bname[j + 1] = j_bname[j]
				j_bname[i] = h
				break
			}
		}
		# Limit to top N
		if (j_bname_n > j_bname_max) {
			j_bname_n = j_bname_max
			j_min_bogon = bogons[j_bname[j_bname_n]]
		}
	}
	next
}

$1 == "group" {
	a = $2
	groups[a] = $3
	arts = $4
	if (arts > a_min_article) {
		a_article[a] = arts
		# Insert in order
		a_name_n++
		for (i = 1; i <= a_name_n; i++) {
			if (a_article[a_name[i]] < arts) {
				# Insert before this guy
				for (j = a_name_n; j >= i; j--)
					a_name[j + 1] = a_name[j]
				a_name[i] = a
				break
			}
		}
		# Limit to top N
		if (a_name_n > a_name_max) {
			a_name_n = a_name_max
			a_min_article = a_article[a_name[a_name_n]]
		}
	}
	next
}

$1 == "groupjunk" {
	group = $2
	junk = $3
	if (junk > j_min_group) {
		j_group[group] = junk
		# Insert in order
		j_name_n++
		for (i = 1; i <= j_name_n; i++) {
			if (j_group[j_name[i]] < junk) {
				# Insert before this guy
				for (j = j_name_n; j >= i; j--)
					j_name[j + 1] = j_name[j]
				j_name[i] = group
				break
			}
		}
		# Limit to top N
		if (j_name_n > j_name_max) {
			j_name_n = j_name_max
			j_min_group = j_group[j_name[j_name_n]]
		}
	}
	next
}

$1 == "peerjunk" {
	hierarchy = $2
	peer = $3
	x = hierarchy "-" peer
	junk = $4
	if (junk > j_min_peer) {
		j_hierarchy[x] = hierarchy
		j_peer[x] = junk
		j_peermap[x] = peer
		# Insert in order
		j_pname_n++
		for (i = 1; i <= j_pname_n; i++) {
			if (j_peer[j_pname[i]] < junk) {
				# Insert before this guy
				for (j = j_pname_n; j >= i; j--)
					j_pname[j + 1] = j_pname[j]
				j_pname[i] = x
				break
			}
		}
		# Limit to top N
		if (j_pname_n > j_pname_max) {
			j_pname_n = j_pname_max
			j_min_peer = j_peer[j_pname_n]
		}
	}
	next
}

{
	print
}

END {
	#### Servers (inbound)

	# Sort 'em
	n = 0
	for (s in servers)
		if (s != t)
			name[++n] = s

	fmt1 = "%-20s %6s %4s %6s %6s %6s %6s %3s %6s %6s\n"
	fmt2 = "%-20.20s %6d %3d%% %6d %6d %6d %6d %3s %6.2f %6.2f\n"
	if (n > 0) {
		name[++n] = t
		printf "Article Reception (they contact us):\n"
		printf fmt1, "", "", "", "", "", "", "", "", "took/", "offer/"
		printf fmt1, "System", "Took", "Pct", "Offer", \
		    "Refuse", "Fail", "Conn", "Ela", "min ", "min "
	}

	# bubblesort (all but the last entry)
	more = n - 1
	while (more) {
		k = more - 1
		more = 0
		for (j = 1; j <= k; j++)
			if (s_received[name[j]] < s_received[name[j + 1]]) {
				s = name[j]
				name[j] = name[j + 1]
				name[j + 1] = s
				more = j
			}
	}

	tot = s_received[t]
	for (i = 1; i <= n; i++) {
		s = name[i]
		if (s == t)
			printf "\n"

		if (tot == 0)
			pct = 100
		else
			pct = (s_received[s] * 100) / tot

		offer = s_received[s] + s_refused[s] + s_rejected[s]

		x = s_elapsed[s]
		if (x < 60)
			ela = int(x) "s"
		else if ((x /= 60) < 60)
			ela = int(x) "m"
		else if ((x /= 60) < 60)
			ela = int(x) "h"
		else
			ela = int(x /= 24) "d"

		if (s_elapsed[s] == 0) {
			trate = 0
			orate = 0
		} else {
			trate = (s_received[s] * 60) / s_elapsed[s]
			orate = (offer * 60) / s_elapsed[s]
		}

		printf fmt2, s, s_received[s], pct, offer, s_refused[s], \
		    s_rejected[s], servers[s], ela, trate, orate
	}
	if (n > 0)
		printf "\n"

	#### Peers (outbound)

	# Sort 'em
	n = 0
	for (s in peers)
		if (s != t)
			name[++n] = s

	fmt1 = "%-20s %6s %4s %6s %6s %6s %6s %3s %6s %6s\n"
	fmt2 = "%-20.20s %6d %3d%% %6d %6d %6d %6d %3s %6.2f %6.2f\n"
	if (n > 0) {
		name[++n] = t
		printf "Article Transmission (we contact them):\n"
		printf fmt1, "", "", "", "", "", "", "", "", "took/", "offer/"
		printf fmt1, "System", "Took", "Pct", "Offer", \
		    "Refuse", "Fail", "Conn", "Ela", "min ", "min "
	}

	# bubblesort (all but the last entry)
	more = n - 1
	while (more) {
		k = more - 1
		more = 0
		for (j = 1; j <= k; j++)
			if (p_accept[name[j]] < p_accept[name[j + 1]]) {
				s = name[j]
				name[j] = name[j + 1]
				name[j + 1] = s
				more = j
			}
	}

	tot = p_accept[t]
	for (i = 1; i <= n; i++) {
		s = name[i]
		if (s == t)
			printf "\n"

		if (tot == 0)
			pct = 100
		else
			pct = (p_accept[s] * 100) / tot

		# XXX should fix nntplink since p_o_rate is also wrong
		offer = p_accept[s] + p_reject[s] + p_failed[s]

		if (p_elapsed[s] == 0) {
			arate = 0
			orate = 0
		} else {
			arate = (p_accept[s] * 60) / p_elapsed[s]
			orate = (offer * 60) / p_elapsed[s]
		}

		x = p_elapsed[s]
		if (x < 60)
			ela = int(x) "s"
		else if ((x /= 60) < 60)
			ela = int(x) "m"
		else if ((x /= 60) < 60)
			ela = int(x) "h"
		else
			ela = int(x /= 24) "d"

		printf fmt2, s, p_accept[s], pct, offer, p_reject[s], \
		    p_failed[s], p_connects[s], ela, arate, orate
	}
	if (n > 0)
		printf "\n"

	#### More peers stuff

	# Calculate total errors
	for (s in peers) {
		t_errors[s] = \
		    t_refused[s] + t_timeout[s] + t_discont[s] + t_denied[s]
	}

	# Sort 'em
	n = 0
	for (s in peers)
		if (s != t && t_errors[s] > 0)
			name[++n] = s

	fmt1 = "%-20s %5s %5s %5s %5s %5s %5s\n"
	fmt2 = "%-20.20s %5d %5d %5d %5d %5d %5d\n"
	if (n > 0) {
		name[++n] = t
		printf "\nTransmission Connection Errors:\n"
		printf fmt1, "System", "Total", \
		    "NS", "Timed", "Ref", "Disc", "Deny"
	}

	# bubblesort (all but the last entry)
	more = n - 1
	while (more) {
		k = more - 1
		more = 0
		for (j = 1; j <= k; j++)
			if (t_errors[name[j]] < t_errors[name[j + 1]]) {
				s = name[j]
				name[j] = name[j + 1]
				name[j + 1] = s
				more = j
			}
	}

	for (i = 1; i <= n; i++) {
		s = name[i]
		if (s == t)
			printf "\n"
		printf fmt2, s, t_errors[s], t_ns[s], t_timeout[s], \
		    t_refused[s], t_discont[s], t_denied[s]
	}
	if (n > 0)
		printf "\n"

	#### Readers

	# Sort 'em
	n = 0
	for (s in readers)
		if (s != t)
		    name[++n] = s

	fmt1 = "%-20s %6s %4s %5s %4s %5s %5s %3s %3s\n"
	fmt2 = "%-20.20s %6d %3d%% %5d %4d %5d %5d %3s %3s\n"
	if (n > 0) {
		name[++n] = t
		printf "Readership statistics:\n"
		printf fmt1, "System", "Arts", "Pct", "Grps", \
		    "Conn", "Post", "Timed", "Ela", "CPU"
	}

	# bubblesort (all but the last entry)
	more = n - 1
	while (more) {
		k = more - 1
		more = 0
		for (j = 1; j <= k; j++)
			if (r_article[name[j]] < r_article[name[j + 1]]) {
				s = name[j]
				name[j] = name[j + 1]
				name[j + 1] = s
				more = j
			}
	}

	tot = r_article[t]
	for (i = 1; i <= n; i++) {
		s = name[i]
		if (s == t)
			printf "\n"

		if (tot == 0)
			pct = 100
		else
			pct = (r_article[s] * 100) / tot

		x = r_elapsed[s]
		if (x < 60)
			ela = int(x) "s"
		else if ((x /= 60) < 60)
			ela = int(x) "m"
		else if ((x /= 60) < 60)
			ela = int(x) "h"
		else
			ela = int(x /= 24) "d"

		x = r_user[s] + r_system[s]
		if (x < 60)
			cpu = int(x) "s"
		else if ((x /= 60) < 60)
			cpu = int(x) "m"
		else if ((x /= 60) < 60)
			cpu = int(x) "h"
		else
			cpu = int(x /= 24) "d"

		printf fmt2, s, r_article[s], pct, r_group[s], r_connects[s], \
		    r_post[s], r_timeout[s], ela, cpu
	}
	if (n > 0)
		printf "\n"

	#### Explorers

	# Sort 'em
	n = 0
	for (s in explorers)
		name[++n] = s

	fmt1 = "%-4s %s\n"
	fmt2 = "%-4d %s%s\n"
	if (n > 0) {
		printf "Unknown nntp Explorers:\n"
		printf fmt1, "Conn", "System"
	}

	# bubblesort
	more = n
	while (more) {
		k = more - 1
		more = 0
		for (j = 1; j <= k; j++)
			if (name[j] > name[j + 1]) {
				s = name[j]
				name[j] = name[j + 1]
				name[j + 1] = s
				more = j
			}
	}

	for (i = 1; i <= n; i++) {
		s = name[i]
		denied = ""
		if (e_denied[s] > 0)
			denied = " (denied)"
		printf fmt2, explorers[s], s, denied
	}
	if (n > 0)
		printf "\n"

	#### Junked groups
	if (j_name_n > 0)
		printf "Junked newsgroups (top %d):\n", j_name_n
	fmt = "%-5d %s\n"
	for (i = 1; i <= j_name_n; i++) {
		s = j_name[i]
		printf fmt, j_group[s], s
	}
	if (j_name_n > 0)
		printf "\n"

	#### Bogon generators
	if (j_bname_n > 0)
		printf "Sites generating bogus headers (top %d):\n", j_bname_n
	fmt = "%-5d %s\n"
	for (i = 1; i <= j_bname_n; i++) {
		s = j_bname[i]
		printf fmt, bogons[s], s
	}
	if (j_bname_n > 0)
		printf "\n"

	#### Junked peers
	if (j_pname_n > 0)
		printf "Junked hierarchy vs. peers (top %d):\n", j_pname_n
	# Width of widest hierarchy
	n = length(sprintf("%d", j_peer[j_pname[1]]))
	m = 4
	for (i = 1; i <= j_pname_n; i++) {
		s = j_pname[i]
		j = length(j_hierarchy[s])
		if (j > m)
			m = j
	}
	fmt = sprintf("%%-%dd %%%ds %%s\n", n, m)
	for (i = 1; i <= j_pname_n; i++) {
		s = j_pname[i]
		printf fmt, j_peer[s], j_hierarchy[s], j_peermap[s]
	}
	if (j_pname_n > 0)
		printf "\n"

	#### Newgroups
	if (a_name_n > 0)
		printf "Article Request Counts (top %d):\n", a_name_max
	fmt = "%-7d %s\n"

	for (i = 1; i <= a_name_n; i++) {
		s = a_name[i]
		printf fmt, a_article[s], s
	}
}
