# @(#) $Header: syslog.awk,v 1.15 92/02/02 16:25:38 leres Exp $ (LBL)
#
# syslog.awk - produce raw data from netnews syslog files

BEGIN {
	t = "TOTALS"
}

$5 ~ "^nnrpd" {
	if ($7 == "connect") {
		r_connects[$6]++
		r_connects[t]++
		next
	}
	if ($7 == "group") {
		readers[$6] = 1
		readers[t] = 1
		groups[$8]++
		g_article[$8] += $9
		next
	}
	if ($7 == "times") {
		r_user[$6] += $9
		r_user[t] += $9
		r_system[$6] += $11
		r_system[t] += $11
		r_elapsed[$6] += $13
		r_elapsed[t] += $13
		next
	}
	if ($7 == "exit") {
		r_article[$6] += $9
		r_article[t] += $9
		r_group[$6] += $11
		r_group[t] += $11
		next
	}
	if ($7 == "timeout") {
		r_timeout[$6]++
		r_timeout[t]++
		next
	}
	if ($7 == "posts") {
		r_post[$6] += $9
		r_post[t] += $9
		# XXX how to count these
#		r_reject[$6] += $11
#		r_reject[t] += $11
		next
	}
	if ($7 == "no" && $8 == "permission") {
		e_denied[$6]++
		next
	}
	if ($7 == "post" && ($8 == "ok" || $8 == "failed"))
		next
	if ($7 == "rejected")
		next
	# XXX should count this?
	if ($7 == "Can't") {
		if ($8 == "read," && $10 == "reset" && $12 == "peer")
			next
	}
	if ($7 == "Can't") {
		if ($8 == "read,") {
			if ($10 == "timed" || $10 == "reset")
				next
		}
	}
}

$5 == "innd:" {
	if ($7 == "connected") {
		# add to server list
		s_connects[$6]++
		s_connects[t]++
		next
	}
	if ($7 == "closed") {
		# Only nntp channels have stats
		if ($8 == "seconds") {
			s_elapsed[$6] += $9
			s_elapsed[t] += $9
			s_received[$6] += $11
			s_received[t] += $11
			s_refused[$6] += $13
			s_refused[t] += $13
			s_rejected[$6] += $15
			s_rejected[t] += $15
		}
		next
	}
	if ($7 == "j") {
		# Copy to the error section if there the wrong number of fields
		if (NF != 9)
			print "error " $0

		# Split 'em
		ofs = FS
		FS = ","
		n = split($9, r)
		FS = ofs

		# Count 'em
		for (i = 1; i <= n; i++) {
			# Junked group
			x = r[i]
			j_group[x]++

			# Hierarchy vs. peer
			j = index(x, ".")
			if (j > 0)
				x = substr(x, 1, j - 1)
			j_peer[$6] = 1
			j_wonk[$6 "-" x]++
			j_hierarchy[x] = 1
		}
		next
	}
	if ($7 == "-") {
		# Extract remote site from message-id (if possible)
		h = $8
		if (h ~ "^<")
			h = substr(h, 2)
		i = index(h, "@")
		# Skip if no @
		if (i == 0)
			next
		# Must look for multiple @'s
		while (i = index(h, "@"))
			h = substr(h, i + 1)
		if (h ~ ">$") {
			i = length(h)
			h = substr(h, 1, i - 1)
		}
		# Skip if no empty
		if (h == "")
			next

		if ($9 == 437) {
			if ($10 == "Bad" && $12 == "header") {
				if ($11 == "\"Message-ID\"") {
					bogons[h]++
					next
				}
				# These are usually human generated
				if ($11 == "\"Expires\"")
					next
				if ($11 == "\"Date\"") {
					bogons[h]++
					next
				}
			}
			if ($10 == "Missing" && $12 == "header") {
				bogons[h]++
				next
			}
			if ($10 == "No" && $11 == "colon-space") {
				bogons[h]++
				next
			}
			if ($10 == "Duplicate") {
				if ($12 == "header") {
					bogons[h]++
					next
				}
				# XXX Should we care about duplicate articles?
				if (NF == 10)
					next
			}
			if ($10 == "Space" && $13 == "header") {
				bogons[h]++
				next
			}
			if ($10 == "No" && $11 == "body") {
				bogons[h]++
				next
			}
			if ($10 == "Article" && $14 == "future")
				next
		}
		if ($10 == "wants" && $12 == "cancel")
			next
	}
	# XXX should count this?
	if ($7 == "cant") {
		if ($8 == "read") {
			if ($10 == "timed" || $10 == "reset")
				next
		}
	}
	if ($7 == "newgroup" || $7 == "rmgroup")
		next
	if ($7 == "running")
		next
	if ($7 == "opened")
		next
	if ($7 == "spawned")
		next
	if ($7 == "flush_all" || $7 == "flush")
		next
	if ($7 == "throttled")
		next
	if ($7 == "paused")
		next
	if ($7 == "shutdown")
		next
	if ($7 == "ccsetup" || $7 == "lcsetup" || $7 == "rcsetup")
		next
	if ($7 == "starting")
		next
	if ($7 == "exit")
		next
	if ($7 == "reload")
		next
	if ($7 == "checkpoint")
		next
	if ($7 == "inactive")
		next
	if ($7 == "restarted")
		next
	if ($7 == "descriptors")
		next
	if ($7 == "readclose")
		next
	if ($7 == "xexec")
		next
}

$5 ~ "^nntplink" {
	h = $6
	# Strip possible trailing colon
	i = length(h)
	if (substr(h, i, 1) == ":")
		h = substr(h, 1, i - 1)
	if ($7 == "stats") {
		# Add peer to the list
		peers[h] = 1
		peers[t] = 1
		p_offer[h] += $8
		p_offer[t] += $8
		p_accept[h] += $10
		p_accept[t] += $10
		p_reject[h] += $12
		p_reject[t] += $12
		p_failed[h] += $14
		p_failed[t] += $14
		# Collect connects if nntplink has been hacked to report them
		if (NF >= 16) {
			p_connects[h] += $16
			p_connects[t] += $16
		}
		next
	}
	if ($7 == "xmit") {
		p_user[h] += $9
		p_user[t] += $9
		p_system[h] += $11
		p_system[t] += $11
		p_elapsed[h] += $13
		p_elapsed[t] += $13
		next
	}
	if ($7 == "xfer") {
#		p_rate[h]++
#		p_rate[t]++
#		p_o_rate[h] += $9
#		p_o_rate[t] += $9
#		p_a_rate[h] += $11
#		p_a_rate[t] += $11
		next
	}
	if ($7 == "socket():") {
		if ($8 == "Connection") {
			if ($9 == "timed") {
				t_timeout[h]++
				t_timeout[t]++
				next
			}
			if ($9 == "refused") {
				t_refused[h]++
				t_refused[t]++
				next
			}
		}
	}
	if ($7 == "host" && $8 == "unknown") {
		t_ns[h]++
		t_ns[t]++
		next
	}
	if ($7 == "greeted") {
		if ($10 == 400) {
			# Service discontinued
			t_discont[h]++
			t_discont[t]++
			next
		}
		if ($10 == 502) {
			# access restriction or permission denied
			t_denied[h]++
			t_denied[t]++
			next
		}
	}
	# XXX should count this?
	if ($7 == "bad" && $8 == "reply:")
		next
	# XXX should count this?
	if ($7 == "article" && $9 == "failed(saved):")
		next
	# XXX should count this?
	if ($7 == "sent" && $8 == "IHAVE" && $10 == "received:") {
		if ($11 == 436)
			next
	}
	# XXX should count this?
	if ($7 == "error" && $8 == "reading" && $9 == "reply:") {
		if ($11 == "reset")
			next
	}
	# XXX should count this?
	if ($7 == "connection" && $8 == "timed" && \
	    $11 == "reading" && $12 == "reply")
		next
}

$7 == ">" || $7 == "<" || $7 == "debug" || $7 == "trace" {
	# Inn conversation debugging
	next
}

$6 == "->" || $6 == "<-" || $6 == "debug" {
	# Other conversation debugging
	next
}

$5 == "syslog:" {
	if ($6 == "rotating" && $7 == "logfiles")
		next
}

# Errors
{
	print "error " $0
}

END {
	# Servers (inbound)
	fmt = "in %s %d %d %d %d %d\n"
	for (s in s_connects)
		if (s_received[s] > 0 || s_refused[s] > 0 || s_rejected[s] > 0)
			printf fmt, s, s_connects[s], s_received[s], \
			    s_refused[s], s_rejected[s], s_elapsed[s]

	# Peers (outbound)
	fmt1 = "out %s %d %d %d %d %d %.3f %.3f %.3f\n"
	fmt2 = "trans %s %d %d %d %d %d\n"
	for (s in peers) {
		if (p_offer[s] != 0 || p_accept[s] != 0 || \
		    p_reject[s] != 0 || p_failed[s] != 0 || p_connects[s])
			printf fmt1, s, p_offer[s], p_accept[s], p_reject[s], \
			    p_failed[s], p_connects[s], \
			    p_user[s], p_system[s], p_elapsed[s]
		if (t_ns[s] != 0 || t_timeout[s] != 0 || t_refused[s] != 0 || \
		    t_discont[s] != 0 || t_denied[s])
			printf fmt2, s, t_ns[s], t_timeout[s], t_refused[s], \
			    t_discont[s], t_denied[s]
	}

	# Readers
	fmt = "read %s %d %d %d %d %d %.3f %.3f %.3f\n"
	for (s in readers)
		printf fmt, s, r_connects[s], r_article[s], r_timeout[s], \
		    r_post[s], r_group[s], r_user[s], r_system[s], r_elapsed[s]

	# Explorers
	fmt = "explore %s %d %d\n"
	for (s in r_connects)
		if (readers[s] == 0)
			printf fmt, s, r_connects[s], e_denied[s]
	for (s in s_connects)
		if (s_received[s] == 0 && s_refused[s] == 0 && \
		    s_rejected[s] == 0)
			printf fmt, s, s_connects[s], 0

	# Sites generating bogus articles
	fmt = "bogon %s %d\n"
	for (s in bogons)
		printf fmt, s, bogons[s]

	# Groups
	fmt = "group %s %d %d\n"
	for (s in groups)
		printf fmt, s, groups[s], g_article[s]

	# Junked groups
	fmt = "groupjunk %s %d\n"
	for (s in j_group)
		printf fmt, s, j_group[s]

	# Junked peers
	fmt = "peerjunk %s %s %d\n"
	for (p in j_peer)
		for (h in j_hierarchy)
			if ((i = j_wonk[p "-" h]) > 0)
			    printf fmt, h, p, i
}
